@@ -1,8 +1,11 @@
 """Functional interface to hypergraph methods and assorted utilities."""
 
 from collections import Counter
+from copy import deepcopy
+from warnings import warn
 
-from ..exception import XGIError
+from ..exception import IDNotFound, XGIError
+from .hypergraph import Hypergraph
 
 __all__ = [
     "max_edge_order",
@@ -21,6 +24,7 @@ __all__ = [
     "get_edge_attributes",
     "is_empty",
     "maximal_simplices",
+    "convert_labels_to_integers",
 ]
 
 
@@ -134,17 +138,15 @@ def edge_neighborhood(H, n, include_self=False):
     >>> H.nodes.neighbors(3)
     {1, 2, 4}
     >>> xgi.edge_neighborhood(H, 3)
-    [[1, 2], [4]]
+    [{1, 2}, {4}]
     >>> xgi.edge_neighborhood(H, 3, include_self=True)
-    [[1, 2, 3], [3, 4]]
+    [{1, 2, 3}, {3, 4}]
 
     """
     if include_self:
         return [H.edges.members(e) for e in H.nodes.memberships(n)]
     else:
-        return [
-            [x for x in H.edges.members(e) if x != n] for e in H.nodes.memberships(n)
-        ]
+        return [H.edges.members(e) - {n} for e in H.nodes.memberships(n)]
 
 
 def degree_counts(H):
@@ -422,13 +424,13 @@ def set_node_attributes(H, values, name=None):
     """
     # Set node attributes based on type of `values`
     if name is not None:  # `values` must not be a dict of dict
-        try:  # `values` is a dict
+        if isinstance(values, dict):  # `values` is a dict
             for n, v in values.items():
                 try:
                     H._node_attr[n][name] = v
-                except KeyError:
-                    pass
-        except AttributeError:  # `values` is a constant
+                except IDNotFound:
+                    warn(f"Node {n} does not exist!")
+        else:  # `values` is a constant
             for n in H:
                 H._node_attr[n][name] = values
     else:  # `values` must be dict of dict
@@ -436,8 +438,8 @@ def set_node_attributes(H, values, name=None):
             for n, d in values.items():
                 try:
                     H._node_attr[n].update(d)
-                except KeyError:
-                    pass
+                except IDNotFound:
+                    warn(f"Node {n} does not exist!")
         except (TypeError, ValueError, AttributeError):
             raise XGIError("Must pass a dictionary of dictionaries")
 
@@ -462,6 +464,7 @@ def get_node_attributes(H, name=None):
     set_node_attributes
     set_edge_attributes
     get_edge_attributes
+
     """
     if name is None:
         return dict(H._node_attr)
@@ -499,26 +502,27 @@ def set_edge_attributes(H, values, name=None):
     -----
     Note that if the dict contains edge IDs that are not in `H`, they are
     silently ignored.
+
     """
     if name is not None:
         # `values` does not contain attribute names
         try:
-            for id, value in values.items():
+            for e, value in values.items():
                 try:
                     H._edge_attr[id][name] = value
-                except KeyError:
-                    pass
+                except IDNotFound:
+                    warn(f"Edge {e} does not exist!")
         except AttributeError:
             # treat `values` as a constant
-            for id in H.edges:
-                H._edge_attr[id][name] = values
+            for e in H.edges:
+                H._edge_attr[e][name] = values
     else:
         try:
-            for id, d in values.items():
+            for e, d in values.items():
                 try:
-                    H._edge_attr[id].update(d)
-                except KeyError:
-                    pass
+                    H._edge_attr[e].update(d)
+                except IDNotFound:
+                    warn(f"Edge {e} does not exist!")
         except AttributeError:
             raise XGIError(
                 "name property has not been set and a dict-of-dicts has not been provided."
@@ -613,3 +617,51 @@ def maximal_simplices(SC):
         if maximal:
             max_simplices.append(i)
     return max_simplices
+
+
+def convert_labels_to_integers(H, label_attribute="label"):
+    """Relabel node and edge IDs to be sequential integers.
+
+    Parameters
+    ----------
+    H : Hypergraph
+        The hypergraph of interest
+
+    label_attribute : string, default: "label"
+        The attribute name that stores the old node and edge labels
+
+    Returns
+    -------
+    Hypergraph
+        A new hypergraph with nodes and edges with sequential IDs starting at 0.
+        The old IDs are stored in the "label" attribute for both nodes and edges.
+
+    Notes
+    -----
+    The "relabeling" will occur even if the node/edge IDs are sequential.
+    Because the old IDs are stored in the "label" attribute for both nodes and edges,
+    the old "label" values (if they exist) will be overwritten.
+    """
+    node_dict = dict(zip(H.nodes, range(H.num_nodes)))
+    edge_dict = dict(zip(H.edges, range(H.num_edges)))
+    temp_H = Hypergraph()
+    temp_H._hypergraph = deepcopy(H._hypergraph)
+
+    temp_H.add_nodes_from((id, deepcopy(H.nodes[n])) for n, id in node_dict.items())
+    set_node_attributes(
+        temp_H, {n: {label_attribute: id} for id, n in node_dict.items()}
+    )
+
+    temp_H.add_edges_from(
+        (
+            {node_dict[n] for n in e},
+            edge_dict[id],
+            deepcopy(H.edges[id]),
+        )
+        for id, e in H.edges.members(dtype=dict).items()
+    )
+    set_edge_attributes(
+        temp_H, {e: {label_attribute: id} for id, e in edge_dict.items()}
+    )
+
+    return temp_H
