@@ -5,10 +5,12 @@ allow modification.  This module provides View classes for nodes, edges, degree,
 edge size of a hypergraph.  Views are automatically updaed when the hypergraph changes.
 
 """
+
+from collections import defaultdict
 from collections.abc import Mapping, Set
 
-from xgi.exception import IDNotFound, XGIError
-from xgi.stats import EdgeStatDispatcher, NodeStatDispatcher
+from ..exception import IDNotFound, XGIError
+from ..stats import EdgeStatDispatcher, NodeStatDispatcher
 
 __all__ = [
     "NodeView",
@@ -176,8 +178,8 @@ class IDView(Mapping, Set):
 
         Parameters
         ----------
-        stat : str
-            Name of a `NodeStat`.
+        stat : str or :class:`xgi.stats.NodeStat`
+            `NodeStat` object, or name of a `NodeStat`.
         val : Any
             Value of the statistic.  Usually a single numeric value.  When mode is
             'between', must be a tuple of exactly two values.
@@ -200,28 +202,44 @@ class IDView(Mapping, Set):
 
         Examples
         --------
+        By default, return the IDs whose value of the statistic is exactly equal to
+        `val`.
+
         >>> import xgi
         >>> H = xgi.Hypergraph([[1, 2, 3], [2, 3, 4, 5], [3, 4, 5]])
-        >>> H.nodes.filterby('degree', 2, 'eq')
+        >>> n = H.nodes
+        >>> n.filterby('degree', 2)
         NodeView((2, 4, 5))
-        >>> H.nodes.filterby('degree', 2, 'neq')
+
+        Can choose other comparison methods via `mode`.
+
+        >>> n.filterby('degree', 2, 'eq')
+        NodeView((2, 4, 5))
+        >>> n.filterby('degree', 2, 'neq')
         NodeView((1, 3))
-        >>> H.nodes.filterby('degree', 2, 'lt')
+        >>> n.filterby('degree', 2, 'lt')
         NodeView((1,))
-        >>> H.nodes.filterby('degree', 2, 'gt')
+        >>> n.filterby('degree', 2, 'gt')
         NodeView((3,))
-        >>> H.nodes.filterby('degree', 2, 'leq')
+        >>> n.filterby('degree', 2, 'leq')
         NodeView((1, 2, 4, 5))
-        >>> H.nodes.filterby('degree', 2, 'geq')
+        >>> n.filterby('degree', 2, 'geq')
         NodeView((2, 3, 4, 5))
-        >>> H.nodes.filterby('degree', (2, 3), 'between')
+        >>> n.filterby('degree', (2, 3), 'between')
         NodeView((2, 3, 4, 5))
 
+        Can also pass a :class:`NodeStat` object.
+
+        >>> n.filterby(n.degree(order=2), 2)
+        NodeView((3,))
+
         """
-        try:
-            stat = getattr(self._dispatcher, stat)
-        except AttributeError as e:
-            raise AttributeError(f'Statistic with name "{stat}" not found') from e
+        if not isinstance(stat, self._dispatcher.statsclass):
+            try:
+                stat = getattr(self._dispatcher, stat)
+            except AttributeError as e:
+                raise AttributeError(f'Statistic with name "{stat}" not found') from e
+
         values = stat.asdict()
         if mode == "eq":
             bunch = [idx for idx in self if values[idx] == val]
@@ -337,11 +355,111 @@ class IDView(Mapping, Set):
         {2}
         >>> H.nodes.neighbors(2)
         {1, 3, 4}
+
         """
         return {i for n in self._id_dict[id] for i in self._bi_id_dict[n]}.difference(
             {id}
         )
 
+    def duplicates(self):
+        """Find IDs that have a duplicate.
+
+        An ID has a 'duplicate' if there exists another ID with the same bipartite
+        neighbors.
+
+        Returns
+        -------
+        IDView
+            A view containing only those IDs with a duplicate.
+
+        Raises
+        ------
+        TypeError
+            When IDs are of different types. For example, ("a", 1).
+
+        Notes
+        -----
+        The IDs returned are in an arbitrary order, that is duplicates are not
+        guaranteed to be consecutive. For IDs with the same bipartite neighbors,
+        only the first ID added is not a duplicate.
+
+        See Also
+        --------
+        IDView.lookup
+
+        Examples
+        --------
+        >>> import xgi
+        >>> H = xgi.Hypergraph([[0, 1, 2], [3, 4, 2], [0, 1, 2]])
+        >>> H.edges.duplicates()
+        EdgeView((2,))
+
+        Order does not matter:
+
+        >>> H = xgi.Hypergraph([[2, 1, 0], [0, 1, 2]])
+        >>> H.edges.duplicates()
+        EdgeView((1,))
+
+        Repetitions matter:
+
+        >>> H = xgi.Hypergraph([[0, 1], [1, 0]])
+        >>> H.edges.duplicates()
+        EdgeView((1,))
+
+        """
+        dups = []
+        hashes = defaultdict(list)
+        for idx, members in self._id_dict.items():
+            hashes[frozenset(members)].append(idx)
+        for _, edges in hashes.items():
+            if len(edges) > 1:
+                dups.extend(sorted(edges)[1:])
+        return self.__class__.from_view(self, bunch=dups)
+
+    def lookup(self, neighbors):
+        """Find IDs with the specified bipartite neighbors.
+
+        Parameters
+        ----------
+        neighbors : Iterable
+            An iterable of IDs.
+
+        Returns
+        -------
+        IDView
+            A view containing only those IDs whose bipartite neighbors match
+            `neighbors`.
+
+        See Also
+        --------
+        IDView.duplicates
+
+        Examples
+        --------
+        >>> import xgi
+        >>> H = xgi.Hypergraph([[0, 1, 2], [3, 4], [3, 4, 2]])
+        >>> H.edges.lookup([3, 4])
+        EdgeView((1,))
+        >>> H.add_edge([3, 4])
+        >>> H.edges.lookup([3, 4])
+        EdgeView((1, 3))
+
+        Can be used as a boolean check for edge existence:
+
+        >>> if H.edges.lookup([3, 4]): print('An edge with members [3, 4] exists')
+        An edge with members [3, 4] exists
+
+        Can also be used to check for nodes that belong to a particular set of edges:
+
+        >>> H = xgi.Hypergraph([['a', 'b', 'c'], ['a', 'd', 'e'], ['c', 'd', 'e']])
+        >>> H.nodes.lookup([0, 1])
+        NodeView(('a',))
+
+        """
+        sought = set(neighbors)
+        found = [idx for idx, neighbors in self._id_dict.items() if neighbors == sought]
+        return self.__class__.from_view(self, bunch=found)
+
     @classmethod
     def from_view(cls, view, bunch=None):
         """Create a view from another view.
@@ -359,7 +477,7 @@ class IDView(Mapping, Set):
         Returns
         -------
         IDView
-            A view object that is identical to `view` but keeps track of different IDs.
+            A view that is identical to `view` but keeps track of different IDs.
 
         """
         newview = cls(None)
@@ -411,6 +529,9 @@ class NodeView(IDView):
     def memberships(self, n=None):
         """Get the edge ids of which a node is a member.
 
+        Gets all the node memberships for all nodes in the view if n
+        not specified.
+
         Parameters
         ----------
         n : hashable
@@ -418,7 +539,7 @@ class NodeView(IDView):
 
         Returns
         -------
-        list
+        dict of sets if n is None, otherwise a set
             Edge memberships.
 
         Raises
@@ -427,7 +548,47 @@ class NodeView(IDView):
             If `n` is not hashable or if it is not in the hypergraph.
 
         """
-        return self._id_dict.copy() if n is None else self._id_dict[n].copy()
+        return (
+            {key: self._id_dict[key] for key in self}
+            if n is None
+            else self._id_dict[n].copy()
+        )
+
+    def isolates(self, ignore_singletons=True):
+        """Nodes that belong to no edges.
+
+        When ignore_singletons is True (default), a node is considered isolated from the
+        rest of the hypergraph when it is included in no edges of size two or more.  In
+        particular, whether the node is part of any singleton edges is irrelevant to
+        determine whether it is isolated.
+
+        When ignore_singletons is False, a node is isolated only when it is a member of
+        exactly zero edges, including singletons.
+
+        Parameters
+        ----------
+        ignore_singletons : bool, default False
+            Whether to consider singleton edges.
+
+        Returns
+        -------
+        NodeView containing the isolated nodes.
+
+        See Also
+        --------
+        :meth:`EdgeView.singletons`
+
+        """
+        if ignore_singletons:
+            nodes_in_edges = set()
+            for members in self._bi_id_dict.values():
+                if len(members) == 1:
+                    continue
+                nodes_in_edges = nodes_in_edges.union(members)
+            isolates = set(self._id_dict) - nodes_in_edges
+            return self.from_view(self, bunch=isolates)
+        else:
+            return self.filterby("degree", 0)
 
 
 class EdgeView(IDView):
@@ -474,6 +635,8 @@ class EdgeView(IDView):
             Edge members.
         dict (if dtype is dict)
             Edge members.
+        set (if e is not None)
+            Members of edge e.
 
         Raises
         ------
@@ -497,3 +660,17 @@ class EdgeView(IDView):
             raise IDNotFound(f'ID "{e}" not in this view')
 
         return self._id_dict[e].copy()
+
+    def singletons(self):
+        """Edges that contain exactly one node.
+
+        Returns
+        -------
+        EdgeView containing the singleton edges.
+
+        See Also
+        --------
+        :meth:`NodeView.isolates`
+
+        """
+        return self.filterby("size", 1)
