@@ -46,8 +46,8 @@ def convert_to_hypergraph(data, create_using=None):
     data : object to be converted
         Current known types are:
          * a Hypergraph object
-         * list-of-lists
-         * dict-of-lists
+         * list-of-iterables
+         * dict-of-iterables
          * Pandas DataFrame (bipartite edgelist)
          * numpy matrix
          * numpy ndarray
@@ -62,7 +62,7 @@ def convert_to_hypergraph(data, create_using=None):
 
     """
     if data is None:
-        empty_hypergraph(create_using)
+        return empty_hypergraph(create_using)
 
     elif isinstance(data, Hypergraph):
         H = empty_hypergraph(create_using)
@@ -87,6 +87,9 @@ def convert_to_hypergraph(data, create_using=None):
     ):
         from_incidence_matrix(data, create_using)
 
+    else:
+        raise XGIError("Input data has unsupported type.")
+
 
 def convert_to_graph(H):
     """Graph projection (1-skeleton) of the hypergraph H.
@@ -119,8 +122,8 @@ def convert_to_simplicial_complex(data, create_using=None):
     data : object to be converted
         Current known types are:
          * a Hypergraph object
-         * list-of-lists
-         * dict-of-lists
+         * list-of-iterables
+         * dict-of-iterables
          * Pandas DataFrame (bipartite edgelist)
          * numpy matrix
          * numpy ndarray
@@ -135,7 +138,7 @@ def convert_to_simplicial_complex(data, create_using=None):
     """
 
     if data is None:
-        empty_hypergraph(create_using)
+        return empty_hypergraph(create_using)
 
     elif isinstance(data, SimplicialComplex):
         H = empty_simplicial_complex(create_using)
@@ -161,6 +164,8 @@ def convert_to_simplicial_complex(data, create_using=None):
         raise XGIError(
             "Not implemented: construction of a SimplicialComplex from incidence matrix"
         )
+    else:
+        raise XGIError("Input data has unsupported type.")
 
 
 def from_hyperedge_list(d, create_using=None, max_order=None):
@@ -168,7 +173,7 @@ def from_hyperedge_list(d, create_using=None, max_order=None):
 
     Parameters
     ----------
-    d : list of lists
+    d : list of iterables
         A hyperedge list
     create_using : Hypergraph constructor, optional
         The hypergraph to add the edges to, by default None
@@ -200,14 +205,14 @@ def to_hyperedge_list(H):
 
     Returns
     -------
-    list of lists
+    list of sets
         The hyperedge list
 
     See Also
     --------
     from_hyperedge_list
     """
-    return list(H.edges.values())
+    return H.edges.members()
 
 
 def from_hyperedge_dict(d, create_using=None):
@@ -249,7 +254,7 @@ def to_hyperedge_dict(H):
     -------
     dict
         A dictionary where the keys are edge IDs and the values
-        are lists of nodes specifying the edges.
+        are sets of nodes specifying the edges.
 
     See Also
     --------
@@ -293,12 +298,12 @@ def from_bipartite_pandas_dataframe(
     # try to get by labels first
     try:
         d = df[[node_column, edge_column]]
-    except Exception:
+    except KeyError:
         # try to index the labels
         try:
             columns = list(df.columns)
             d = df[[columns[node_column], columns[edge_column]]]
-        except KeyError:
+        except (KeyError, TypeError):
             raise XGIError("Invalid columns specified")
 
     if isinstance(H, SimplicialComplex):
@@ -572,7 +577,7 @@ def dict_to_hypergraph(hypergraph_dict, nodetype=None, edgetype=None):
 
     See Also
     --------
-    read_hypergraph_json
+    read_json
 
     """
     H = empty_hypergraph()
@@ -606,7 +611,7 @@ def dict_to_hypergraph(hypergraph_dict, nodetype=None, edgetype=None):
 
             if nodetype is not None:
                 try:
-                    edge = [nodetype(n) for n in edge]
+                    edge = {nodetype(n) for n in edge}
                 except Exception as e:
                     raise TypeError(
                         f"Failed to convert nodes to type {nodetype}."
