@@ -6,6 +6,7 @@ hypergraph).
 """
 
 from itertools import combinations
+from warnings import warn
 
 import networkx as nx
 
@@ -17,6 +18,7 @@ __all__ = [
     "star_clique",
     "flag_complex",
     "sunflower",
+    "ring_lattice",
 ]
 
 
@@ -209,27 +211,57 @@ def flag_complex(g, max_order=2, seed=None):
     return S
 
 
-def lattice(n, d, k, l):
-    """A lattice hypergraph.
+def ring_lattice(n, d, k, l):
+    """A ring lattice hypergraph.
 
     A d-uniform hypergraph on n nodes where each node is part of k edges and the
     overlap between consecutive edges is d-l.
 
+    Parameters
+    ----------
+    n : int
+        Number of nodes
+    d : int
+        Edge size
+    k : int
+        Number of edges of which a node is a part. Should be a multiple of 2.
+    l : int
+        Overlap between edges
+
+    Returns
+    -------
+    Hypergraph
+        The generated hypergraph
+
+    Raises
+    ------
+    XGIError
+        If k is negative.
+
     Notes
     -----
-
-    lattice(n, 2, k, 0) is a ring lattice graph where each node has k//2 edges on either
+    ring_lattice(n, 2, k, 0) is a ring lattice graph where each node has k//2 edges on either
     side.
-
     """
     from ..classes import Hypergraph
 
+    if k < 0:
+        raise XGIError("Invalid k value!")
+
+    if k < 2:
+        warn("This creates a completely disconnected hypergraph!")
+
+    if k % 2 != 0:
+        warn("k is not divisible by 2")
+
     edges = [
         [node] + [(start + l + i) % n for i in range(d - 1)]
         for node in range(n)
         for start in range(node + 1, node + k // 2 + 1)
     ]
-    return Hypergraph(edges)
+    H = Hypergraph(edges)
+    H.add_nodes_from(range(n))
+    return H
 
 
 def sunflower(l, c, m):
