@@ -49,6 +49,7 @@ from typing import Callable
 
 import numpy as np
 import pandas as pd
+from scipy.stats import moment as spmoment
 
 from xgi.exception import IDNotFound
 
@@ -230,6 +231,10 @@ class IDStat:
         """The minimum value of this stat."""
         return self.asnumpy().min(axis=0)
 
+    def sum(self):
+        """The sum of this stat."""
+        return self.asnumpy().sum(axis=0)
+
     def mean(self):
         """The arithmetic mean of this stat."""
         return self.asnumpy().mean(axis=0)
@@ -246,6 +251,20 @@ class IDStat:
         """The variance of this stat."""
         return self.asnumpy().var(axis=0)
 
+    def moment(self, order=2, center=False):
+        """The statistical moments of this stat.
+
+        Parameters
+        ----------
+        order : int (default 2)
+            The order of the moment.
+        center : bool (default False)
+            Whether to compute the centered (False) or uncentered/raw (True) moment.
+
+        """
+        arr = self.asnumpy()
+        return spmoment(arr, moment=order) if center else np.mean(arr**order)
+
     def dist(self):
         return np.histogram(self.asnumpy(), density=True)
 
